// ACCELOMOTER
#include<Wire.h>
#include <Servo.h>

// accelerometer
const int MPU=0x68; 
int16_t AcX,AcY;

// SERVOS
Servo Servo1;
Servo Servo2;
Servo Servo3;
Servo Servo4;
Servo Servo5;
Servo Servo6;



// fixed parameters, imported from Grasshopper
const float servoLength = 16.0;
const float rodLength = 123.0;
const double beta[6] = {radians(60),radians(-120),radians(180),radians(0),radians(300),radians(120)}; // Servo direction angles

float transformation[6]; // desired transformation
double RM[3][3]; // rotationmatrix
double qT[6][3]; // create array for transformed platform points

int posX = 0;
int posY = 0;
int stepsize = 10;

const float targetX = 0;
const float targetY = 0;

float errorX;
float errorY;

float Kp = 0.06;
int maxAngle = 10;

float PIDphi=0;
float PIDtheta=0;
float PIDpsi=0;
float PIDphi_C=0;
float PIDtheta_C=0;

// function calculating rotation matrix
void getMatrix(float tranformation[]){
  float psi=transformation[3];
  float theta=transformation[4];
  float phi=transformation[5];
  RM[0][0] = cos(psi)*cos(theta);
  RM[1][0] = -sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi);
  RM[2][0] = sin(psi)*sin(phi)+cos(psi)*cos(phi)*sin(theta);

  RM[0][1] = sin(psi)*cos(theta);
  RM[1][1] = cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi);
  RM[2][1] = -cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi);

  RM[0][2] = -sin(theta);
  RM[1][2] = cos(theta)*sin(phi);
  RM[2][2] = cos(theta)*cos(phi);
}

 const float b[6][3] = { //basePoint
{47.5, -54.853, 0},
{71.25, -13.71325, 0},
{23.75, 68.56625, 0},
{-23.75, 68.56625, 0},
{-71.25, -13.71325, 0},
{-47.5, -54.853, 0},
 };

 const float p[6][3] = { // platformPoints
  {29.692614, -85.703383, 0},
{89.067614, 17.137133, 0},
{59.376279, 68.559606, 0},
{-59.381393, 68.564035, 0},
{-89.068893, 17.143777, 0},
{-29.686221, -85.701169, 0},
 };

 double q[6][3] = { // same thing for q, that will be changed
{29.692614, -85.703383, 0},
{89.067614, 17.137133, 0},
{59.376279, 68.559606, 0},
{-59.381393, 68.564035, 0},
{-89.068893, 17.143777, 0},
{-29.686221, -85.701169, 0},
};

 // sqrt(sq(rodLength)+sq(servoLength)-sq(px-bx)-sq(py-by));
const double h0 = sqrt(sq(rodLength)+sq(servoLength)-sq(p[0][0]-b[0][0])-sq(p[0][1]-b[0][1])); // baseHeight
const double l0 = sqrt(sq(p[0][0]-b[0][0])+sq(p[0][1]-b[0][1])+sq(h0+p[0][2]));

// get alpha0
double M0 = 2*servoLength*(cos(beta[0])*(q[0][0]-b[0][0])+sin(beta[0])*(q[0][1]-b[0][1]));
double N0 = 2*servoLength*(p[0][2]+h0);
double L0 = sq(l0)-(sq(rodLength)-sq(servoLength));

double alpha0 = asin(L0/sqrt(sq(M0)+sq(N0)))-atan(M0/N0);

void setup() {


  Wire.begin();
  Wire.beginTransmission(MPU);
  Wire.write(0x6B); 
  Wire.write(0);    
  Wire.endTransmission(true);


  Serial.begin(9600);
// initialise servos

  int x = 90;
  int y = 1000;

  Servo1.attach(2);
  Servo2.attach(3);
  Servo3.attach(4);
  Servo4.attach(5);
  Servo5.attach(6);
  Servo6.attach(7);

  Servo1.write(x);
  delay(y);
  Servo2.write(x);
  delay(y);
  Servo3.write(x);
  delay(y);
  Servo4.write(x);
  delay(y);
  Servo5.write(x);
  delay(y);
  Servo6.write(x);
}




void loop() {

  Wire.beginTransmission(MPU);
  Wire.write(0x3B);  
  Wire.endTransmission(false);
  Wire.requestFrom(MPU,12,true); 
  AcX=Wire.read()<<8|Wire.read();    
  AcY=Wire.read()<<8|Wire.read(); 

  posX = AcX / 100;
  posY = AcY / 100;

  Serial.print("Accelerometer: ");
  Serial.print("X = "); Serial.print(posX);
  Serial.print(" | Y = "); Serial.println(posY);


errorX = targetX - posX;
errorY = targetY - posY;

float testAngleX = errorX*Kp;
float testAngleY = errorY*Kp;

 
// get desired rotation values
transformation[0] = 0;// x-axis
transformation[1] = 0;// y-axis
transformation[2] = 0;// z-axis

transformation[3] = radians(0);// psi
transformation[4] = radians(testAngleY);// theta
transformation[5] = radians(testAngleX);// phi

getMatrix(transformation); // change matrix to specific rotation values

// check if rotation matrix is correct
for(int i=0;i<6;i++){
 qT[i][0] = /*transformation[0]+*/RM[0][0]*(q[i][0])+RM[1][0]*(q[i][1])+RM[2][0]*(q[i][2]);
 qT[i][1] = /*transformation[1]+*/RM[0][1]*(q[i][0])+RM[1][1]*(q[i][1])+RM[2][1]*(q[i][2]);
 qT[i][2] = /*transformation[2]+*/RM[0][2]*(q[i][0])+RM[1][2]*(q[i][1])+RM[2][2]*(q[i][2])+h0;
 }

double lVec[6][3];

for (int i=0; i<6; i++) {
lVec[i][0] = qT[i][0] - b[i][0];
lVec[i][1] = qT[i][1] - b[i][1];
lVec[i][2] = qT[i][2] - b[i][2];
}

double l[6];
for (int i=0; i<6;i++){
 l[i]=sqrt(sq(lVec[i][0])+sq(lVec[i][1])+sq(lVec[i][2]));
}

double M[6];
double N[6];
double L[6];
double alpha[6];
double alphaRes[6];

for (int i=0; i<6; i++){
 M[i] = 2*servoLength*(cos(beta[i])*(qT[i][0]-b[i][0])+sin(beta[i])*(qT[i][1]-b[i][1]));
 N[i] = 2*servoLength*(qT[i][2]);
 L[i] = sq(l[i])-(sq(rodLength)-sq(servoLength));
}

for (int i=0; i<6; i++) {
alpha[i] = asin(L[i]/sqrt(sq(M[i])+sq(N[i])))-atan(M[i]/N[i]);
}

//Serial.println("alpha");
for (int i=0; i<6; i++){
alphaRes[i] = (alpha[i] - alpha0)*57.295779;
  
if(i%2 != 0){
  alphaRes[i] += 90;
  }
else{
  alphaRes[i] = 90 - alphaRes[i];
  }

}


// move servos
Servo1.write(alphaRes[0]);
Servo2.write(alphaRes[1]);
Servo3.write(alphaRes[2]);
Servo4.write(alphaRes[3]);
Servo5.write(alphaRes[4]);
Servo6.write(alphaRes[5]);

}
